from ludvig.scanners import SecretScanner, VulnerabilityScanner, ScanPipeline
from ludvig import Severity
from ludvig.providers import ContainerProvider
from ludvig.config import get_config
from ludvig.vulndb import get_vuln_db


def scan(
    repository: str,
    severity_level: Severity = Severity.MEDIUM,
    enabled=["secret", "vuln"],
    deobfuscated=False,
    output_sarif=None,
    include_first_layer=False,
    max_file_size=10000,
):
    """
    Scans a container image
    :param repository: Container image to scan (ex: myimage:1.1)
    :param severity_level: Set severity level for reporting
    :param deobfuscated: Returns any secrets found in plaintext. Default: False.
    :param output_sarif: Generates SARIF report if filename is specified.
    :param include_first_layer: Scan first layer (base image) as well - may affect speed. Default: False.
    :param max_file_size: Max file size for scanning (in bytes).
    """
    if isinstance(severity_level, str):
        severity_level = Severity[severity_level]
    provider = ContainerProvider(
        repository, include_first_layer, max_file_size=max_file_size
    )
    config = get_config()
    scanners = []
    if "secret" in enabled:
        scanners.append(SecretScanner(config, deobfuscated))
    if "vuln" in enabled:
        scanners.append(VulnerabilityScanner(get_vuln_db(config), config))

    pipeline = ScanPipeline(scanners, provider, severity_level)
    pipeline.scan()
    if output_sarif:
        from ludvig.outputs import SarifConverter

        report = SarifConverter.from_findings(pipeline.findings)
        with open(output_sarif, "w") as r:
            r.write(report)
    return pipeline.findings


# def list_whiteouts(repository: str, include_first_layer=False):
#     """
#     Scans a container image for deleted files
#     :param repository: Container image to scan
#     :param include_first_layer: Scan first layer (base image) as well - may affect speed. Defaults to False.
#     """
#     provider = ContainerProvider(repository, include_first_layer)
#     scanner = ImageScanner(provider)
#     return scanner.list_whiteout()



from enum import IntEnum
import hashlib
import json
from typing import List
import yara
from dataclasses import dataclass, field, asdict
from ludvig.vulndb import Advisory


class Severity(IntEnum):
    UNKNOWN = 0
    LOW = 1
    MEDIUM = 2
    HIGH = 3
    CRITICAL = 4


@dataclass
class RuleMatch:
    rule_id: str
    rule_name: str
    severity: Severity = field(default_factory=lambda: Severity.MEDIUM)
    category: str = field(default=None)
    description: str = field(default=None)
    tags: list[str] = field(default_factory=lambda: [])

    @property
    def __dict__(self):
        return asdict(self)

    @staticmethod
    def from_yara_match(yara: yara.Match) -> "RuleMatch":
        return RuleMatch(
            yara.meta["id"] if "id" in yara.meta else "LS00000",
            yara.rule,
            Severity[yara.meta["severity"]]
            if "severity" in yara.meta
            else Severity.UNKNOWN,
            yara.namespace,
            yara.meta["description"] if "description" in yara.meta else "",
            yara.tags,
        )

    def from_vuln_advisory(advisory: Advisory) -> "RuleMatch":
        return RuleMatch(
            advisory.id,
            advisory.ext_id,
            Severity.HIGH,
            advisory.ecosystem,
            advisory.details,
        )


class FindingSample:
    def __init__(
        self, content: str, offset: int, deobfuscated=False, line_number: int = -1
    ) -> None:
        self.offset = offset
        self.line_number = line_number
        content = content[:10] + "..." if len(content) > 10 else content
        if deobfuscated:
            self.content = content
        else:
            obfuscated = "*" * len(content)
            self.content = (
                obfuscated[:10] + "..." if len(obfuscated) > 10 else obfuscated
            )

    def toJson(self):
        return json.dumps(self, default=lambda o: o.__dict__)

    @classmethod
    def from_yara_match(
        cls, match: yara.Match, deobfuscated=False, line_number: int = -1
    ) -> List["FindingSample"]:
        samples = []
        for str_match in match.strings:
            offset = str_match[0]
            data = str_match[2]
            if data.isascii():
                data = data.decode("utf-8")
            else:
                data = "".join(format(x, "02x") for x in data)
            samples.append(FindingSample(data, offset, deobfuscated, line_number))
        return samples


@dataclass
class Finding:
    id: str
    category: str
    rule: RuleMatch
    filename: str
    severity: Severity = field(init=False)
    samples: list[FindingSample] = field(default_factory=lambda: [])
    properties: dict = field(default_factory=dict)
    _hash: str = field(init=False, repr=False)

    def __post_init__(self):
        self.name = f"{self.category}/{self.rule.rule_name}"
        self.severity = self.rule.severity
        self.properties.update({"category": self.category})
        self._hash = hashlib.sha1(
            "|".join(
                [
                    self.name,
                    self.filename,
                    "@".join([s.content for s in self.samples]),
                ]
            ).encode()
        ).hexdigest()

    @property
    def __dict__(self):
        return asdict(self)

    @staticmethod
    def from_secret(
        yara_match: yara.Match,
        samples: list[FindingSample],
        file_name: str,
        meta: dict = {},
    ) -> "Finding":
        rule = RuleMatch.from_yara_match(yara_match)
        return Finding(rule.rule_id, rule.category, rule, file_name, samples, meta)

    @staticmethod
    def from_vuln_advisory(
        advisory: Advisory, actual_version: str, filename: str, meta: dict = {}
    ) -> "Finding":
        rule = RuleMatch.from_vuln_advisory(advisory)
        return Finding(
            advisory.id,
            "vulnerabilities",
            rule=rule,
            filename=advisory.package.name,
            properties=meta,
        )



import os
from unittest import TestCase
import mock
from ludvig.scanners import VulnerabilityScanner
from ludvig import Severity
from ludvig.config import Config
from ludvig.vulndb import Advisory, Package
import ludvig.vulndb


class TestVulnDb(ludvig.vulndb.VulnDb):
    def __init__(self) -> None:
        pass

    def query_advisory(
        self,
        package_name: str,
        ecosystem: str,
        version: str,
    ):
        if package_name == "Newtonsoft.Json":
            return [
                Advisory(
                    1,
                    "test",
                    None,
                    Package("Newtonsoft.Json"),
                    "nuget",
                    None,
                    "test advisory",
                    "0",
                )
            ]
        return []


class TestVulnDbDotNet(TestCase):
    def test_can_detect_vulnerable_package_in_project(self):
        deps_file = os.path.join(
            os.path.dirname(__file__), "fixtures/dotnet.deps-example.json"
        )
        config = Config("test", rule_sources=[], vulndb_sources=[])
        scanner = VulnerabilityScanner(TestVulnDb(), config)
        with open(deps_file, "r") as f:
            findings = scanner.scan_file_data(f, "deps.json", Severity.MEDIUM)
        self.assertEqual(len(findings), 1)
        self.assertEqual(findings[0].filename, "Newtonsoft.Json")



import json
import os
from dataclasses import dataclass, asdict


def get_config() -> "Config":
    return Config.load()


@dataclass
class VulnDbSource:
    name: str
    uri: str

    @property
    def __dict__(self):
        return asdict(self)


@dataclass
class RuleSetSource:
    name: str
    category: str
    url: str

    @property
    def __dict__(self):
        return asdict(self)


class ConfigEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, RuleSetSource):
            return obj.__dict__
        return json.JSONEncoder.default(self, obj)


class Config:
    def __init__(
        self,
        config_path: str,
        rule_sources: list[RuleSetSource] = None,
        vulndb_sources: list[VulnDbSource] = None,
    ) -> None:
        self.config_path = config_path
        self.compiled_rules = os.path.join(config_path, "ludvig.rules")
        self.config_file = os.path.join(config_path, "config.json")
        if rule_sources and "Built-in" not in rule_sources:
            rule_sources.append(
                RuleSetSource(
                    "Built-In",
                    "secrets",
                    "https://github.com/FrodeHus/ludvig-rules/archive/refs/tags/v0.0.1.tar.gz",
                )
            )
        else:
            rule_sources = [
                RuleSetSource(
                    "Built-In",
                    "secrets",
                    "https://github.com/FrodeHus/ludvig-rules/archive/refs/tags/v0.0.1.tar.gz",
                )
            ]

        self.rule_sources = rule_sources
        if not vulndb_sources:
            vulndb_sources = [
                VulnDbSource(
                    "GitHub Advisory",
                    "https://github.com/github/advisory-database/archive/refs/heads/main.zip",
                )
            ]
        self.vulndb_sources = vulndb_sources
        self.vuln_db_file = os.path.join(config_path, "ludvig.db")

    def save(self):
        with open(self.config_file, "w") as f:
            f.write(json.dumps(self.__dict__, indent=4, cls=ConfigEncoder))

    @staticmethod
    def load():
        config_path = os.path.join(os.path.expanduser("~"), ".ludvig")
        if not os.path.exists(config_path):
            os.makedirs(config_path)
        config_file = os.path.join(config_path, "config.json")
        if os.path.exists(config_file):
            with open(config_file, "r") as f:
                config = json.loads(f.read())
                return Config(
                    config["config_path"],
                    Config.parse_rule_sets(config["rule_sources"]),
                )

        return Config(config_path)

    @staticmethod
    def parse_rule_sets(d: dict):
        rules = []
        for rule_source in d:
            rules.append(
                RuleSetSource(
                    rule_source["name"], rule_source["category"], rule_source["url"]
                )
            )



from knack.log import get_logger
from ludvig.vulndb import VulnDb
from ludvig.config import get_config

logger = get_logger(__name__)


def build():
    """
    Builds the vulnerability database from scratch
    """
    current_config = get_config()
    VulnDb.build(current_config.vuln_db_file, current_config.vulndb_sources)
